/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 *
 * ServiceConfiguration.hxx
 *
 * Authors: Paolo Badino <paolo.badino@cern.ch>
 * Version info: $Id: ServiceConfigurator.hxx,v 1.11 2006/03/09 11:39:08 badino Exp $
 * Release: $Name: glite-data-config-service_R_1_2_7 $
 *
 */
#ifndef GLITE_DATA_CONFIG_SERVICE_SERVICECONFIGURATOR_HXX_
#define GLITE_DATA_CONFIG_SERVICE_SERVICECONFIGURATOR_HXX_

#include <string>
#include <map>
#include <list>
#include "glite/data/config/service/ComponentConfiguration.hxx"

namespace log4cpp{
    class Category;
}

namespace glite {
namespace config{

/**
 * ServiceConfigurator
 * this class read the configuration from a XML file and then dinamically loads
 * and configure all the Components that should compose the Service Application 
 */
class ServiceConfigurator {
public:

    /**
     * ServiceConfigurator return values
     */
    enum {
        SC_FAILED           = -1,
        SC_NOT_INIT         = -2,
        SC_NOT_STARTED      = -3,
        SC_SUCCESS          =  0,
        SC_ALREADY_INIT     =  1,
        SC_ALREADY_STARTED  =  2
    };
    
    /**
     * Return Singleton instance
     * @return Singleton instance
     */
    static ServiceConfigurator * instance();

    /**
     * Deletes singleton instance. If the service is running, it will be stopped
     * and finalized.
     */
    static void dispose();
    
    /**
     * Initialize the Service
     * @param name [IN] the configuration namespace that should be used for that
     * service
     * @return SC_SUCCESS if success, SC_ALREADY_INIT if already initialized or 
     * SC_FAILED if failed
     */
    int init(const char * name);

    /**
     * Initialize the Service
     * @param dir_name [IN] the name of the configuration directory 
     * @param use_as_prefix [IN] use the configuration directory as profix of 
     * the instance name
     * @param instance_name [IN] the name of the configuration instance. The 
     * configuration namespace for the instance is then computed depending on 
     * the value of the use_as_prefix parameter: if set to true the instance 
     * name will be <dir_name>-<name>, otherwise just name
     * @return SC_SUCCESS if success, SC_ALREADY_INIT if already initialized or 
     * SC_FAILED if failed
     */
    int init(const char * dir_name, bool use_as_prefix, const char * instance_name);

    /**
     * Start the Service
     * @return SC_SUCCESS if success, SC_ALREADY_STARTED if already started,
     * SC_NOT_INIT if not initialized or SC_FAILED if failed
     */
    int start();

    /**
     * Stop the Service
     * @return SC_SUCCESS if success, SC_NOT_STARTED if not started, SC_NOT_INIT 
     * if not initialized or SC_FAILED if failed
     */
    int stop();
    
    /**
     * Reconfigure the Service. This method will perform the action 
     * stop-config-start on all the components
     * @return SC_SUCCESS if success, SC_NOT_INIT if not initialized or 
     * SC_FAILED if failed
     */
    int reconfig();
    
    /**
     * Finalize the Service
     * @return SC_SUCCESS if success, SC_NOT_INIT if not initialized or 
     * SC_FAILED if failed
     */
    int fini();

    /**
     * Return Location Path
     * @return Location Path
     */
    const char * getLocation() const  { return m_location.c_str();}

    /**
     * Return Var Location Path
     * @return Var Location Path
     */
    const char * getVarLocation() const  { return m_locationVar.c_str();}

    /**
     * Return User Location Path
     * @return User Location Path
     */
    const char * getUserLocation() const  { return m_locationUser.c_str();}

    /**
     * Register a Component: it will be configured without loading it dynamically
     * @param component [IN] the component to be registered
     * @return 0 if success
     */
    int registerComponent(ComponentConfiguration * component);

    /**
     * Deregister a Static Component
     * @param component [IN] the component to be deregistered
     * @return 0 if success
     */
    int deregisterComponent(ComponentConfiguration * component);
    
    /**
     * Get Locations Variables
     * @param location [OUT] the value of GLITE_LOCATION
     * @param location_var [OUT] the value of GLITE_LOCATION_VAR
     * @param location_user [OUT] the value of GLITE_LOCATION_USER
     */
    static void getLocations(std::string& location, std::string& location_var, std::string& location_user);
    
    /**
     * Initialize variables for changing the process name. This method shoudl be
     * called in the main function if the process is interested in forking child
     * processing and updating their names
     * @param argc [IN] the number of command line arguments
     * @param argv [IN] the command line arguments
     * @return the maximum length of the name that could stored
     */
    static int initProcNameVars(int argc,char **argv);

    /**
     * Change the process name
     * @param name [IN] the name to assign to the process
     * @return the length of the name assigned to the process
     */
    static int setProcName(const char * name);
    
protected:

    /**
     * Constructor
     * @param name [IN] the name of the service
     */
    ServiceConfigurator();
    
    /**
     * Destructor
     */
    ~ServiceConfigurator();
    
    /**
     * Load Components
     * @return 0 if success
     */
    int loadComponents();

    /**
     * Reload Configuration
     * @return 0 if success
     */
    int reloadConfiguration();
    
    /**
     * Initialize Components
     * @return 0 if success
     */
    int initComponents();

    /**
     * Initialize Logger
     * @return 0 if success
     */
    int initLogger();

private:
    
    /**
     * ComponentItem Structure
     */
    struct ComponentItem {
        typedef std::list<void*>        HandleList;
        
        void *                          comp_handle;
        ComponentConfiguration *        component;
        HandleList                      dependencies;
        ComponentConfiguration::Params  init_params;
        ComponentConfiguration::Params  config_params;
    };
    
    typedef std::map<std::string, ComponentItem *> ComponentMap;
    typedef std::list<ComponentItem *>             ComponentList;
    
    /*
     * Dynamically load a Component
     */
    ComponentItem * loadComponent(const char * name, const char * libname, const std::list<std::string>& deplist);
    
    /*
     * Set Location Properties
     */
    void setLocations();

    /*
     * Get the path of a Configuration File
     */
    std::string getFilePath(const std::string& filename);
    
    /*
     * Get the path of a Configuration File using the directory-instance name
     */
    std::string getFilePath(const std::string& dir, const std::string& filename);
    
    /*
     * Get the value of the configuration path form a file
     */
    static void getVarFromFile(const std::string& fname,std::string& location,std::string& location_var,std::string& location_user);

    /*
     * validate instance name
     */
    bool validateName(const char* name);
    
    /*
     * Enter a debug Log in case the logger is not yet configured
     */
    void debugLog(const char * msg);
    
private:

    typedef enum {
        SC_STATUS_IDLE          = 0,
        SC_STATUS_INITIALIZED   = 1,
        SC_STATUS_STARTED       = 2,
        SC_STATUS_FAILED        = 3
    } SCStatus;
    
    /**
     * Service Name
     */
    std::string m_name;    

    /**
     * Service Directory Name
     */
    std::string m_dirName;    
    
    /**
     * Service Instance Name
     */
    std::string m_instanceName;    
    
    /**
     * Service Status
     */
    SCStatus m_status;

    /**
     * The Service Component Map
     */
    ComponentMap m_components;
    
    /**
     * The Service Component List
     */
    ComponentList m_componentList;
    
    /**
     * Location Path
     */
    std::string m_location;

    /**
     * Var Location Path
     */
    std::string m_locationVar;
  
    /**
     * User Location Path
     */
    std::string m_locationUser;
    
    /**
     * The Logger
     */
    class log4cpp::Category& m_logger;

    /**
     * The Singleton instance
     * @label singleton
     */
    static ServiceConfigurator * s_instance;
    
    /*
     * The command line arguments
     */
    static char ** s_argv;
    
    /*
     * The max service name length
     */
    static int s_nameLen;

    /** @link dependency 
     * @label configure*/
    /*# ComponentConfiguration lnkComponentConfiguration; */
};

} // End namespace config
} // End namespace glite

#endif //GLITE_DATA_CONFIG_SERVICE_SERVICECONFIGURATOR_HXX_
